
from __future__ import annotations
from typing import TYPE_CHECKING
from abc import ABC, abstractmethod
from curses import window

if TYPE_CHECKING:
    from cursedtodo.utils.router import Router


class Controller(ABC):
    def __init__(self, router: Router) -> None:
        self.window: window = router.window
        self.router = router

    @abstractmethod
    def handle_key(self, key: int) -> bool: ...



from __future__ import annotations

import curses
from curses import newpad, window
from datetime import datetime
from operator import attrgetter
from typing import TYPE_CHECKING

from cursedtodo.config import Config
from cursedtodo.models.todo import Todo
from cursedtodo.utils.colors import RED, WHITE, random_color
from cursedtodo.utils.formater import Formater
from cursedtodo.utils.time import TimeUtil
from cursedtodo.utils.window_utils import add_borders
from cursedtodo.views.base_view import BaseView

if TYPE_CHECKING:
    from cursedtodo.controlers.main_controller import MainController


class MainView(BaseView):
    def __init__(self, controller: MainController) -> None:
        super().__init__(controller)
        self.controller = controller
        self.index = 0
        self.selected = 0

    def render(self) -> None:
        self.height, self.length = self.window.getmaxyx()
        self.window.erase()
        add_borders(self.window)
        self.window.addstr(0, 5, Config.ui.window_name)
        self.window.addstr(
            self.height - 1,
            5,
            f" q: quit | {chr(Config.keybindings.show_completed)}: show completed | {chr(Config.keybindings.change_order)}: change order | {'space' if chr(Config.keybindings.mark_as_done) == ' ' else chr(Config.keybindings.mark_as_done)}: mark as done | {chr(Config.keybindings.delete)}: delete ",
        )
        self.window.refresh()
        self.pad = newpad(max(len(self.controller.data), self.length), self.length)
        self.render_content()

    def render_line(self, pad: window, y: int, todo: Todo) -> None:
        columns = Config.columns
        pad.move(y, 0)
        for column in columns:
            content = ""
            attr = 0
            if column.property == "priority" and todo.priority > 0:
                content, attr = Formater.formatPriority(todo.priority)
            elif column.property == "due" and todo.due is not None:
                local_tz = TimeUtil.get_locale_tz()
                if todo.due.replace(tzinfo=local_tz) < datetime.now(local_tz):
                    attr = RED
                content = f"{todo.due.strftime(Config.ui.date_format)}"
            elif column.property == "calendar.name":
                content = todo.calendar.name
                attr = todo.calendar.color_attr
            elif column.property == "categories" and todo.categories:
                content = ", ".join(todo.categories or [])
                for category in todo.categories:
                    color = (
                        random_color(category) if Config.ui.category_colors else WHITE
                    )
                    pad.addstr(f"{category} ", color)
                continue
            else:
                getter = attrgetter(column.property)
                content = str(getter(todo) or "")
            pad.addstr(content[: column.width - 1].ljust(column.width), attr)
        if todo.completed:
            pad.chgat(y, 0, self.length, curses.A_DIM)
        if y == self.selected:
            pad.chgat(y, 0, self.length, curses.A_STANDOUT)

    def render_content(self) -> None:
        self.pad.erase()
        self.pad.resize(max(len(self.controller.data), self.length), self.length)
        if self.height - self.selected > self.index:
            self.index = self.selected - self.height + 3
        for i, todo in enumerate(self.controller.data):
            self.render_line(self.pad, i, todo)
        self.pad.refresh(self.index, 0, 1, 1, self.height - 2, self.length - 2)

    def main_loop(self) -> None:
        while True:
            k = self.pad.getch()
            if self.controller.handle_key(k):
                break
            elif (
                k == Config.keybindings.down
                and self.selected < len(self.controller.data) - 1
            ):
                self.selected += 1
                if self.selected > self.height - 3:
                    self.index += 1
            elif k == Config.keybindings.up and self.selected > 0:
                self.selected -= 1
                if self.selected < self.index:
                    self.index -= 1
            self.render_content()



from dataclasses import dataclass
import os
import pathlib
import tomllib
import shutil
from typing import Any, Optional

from cursedtodo.config.arguments import Arguments
from cursedtodo.models.calendar import Calendar


@dataclass
class UIConfig:
    window_name: str
    show_footer_keybindings: bool
    select_first: bool
    rounded_borders: bool
    date_format: str
    default_calendar: Optional[str] = None
    category_colors: bool = False
    confirm_mark_as_done: bool = False


@dataclass
class Columns:
    property: str
    width: int


class KeyBindings:
    up: int
    down: int
    new: int
    delete: int
    edit: int
    mark_as_done: int
    show_completed: int
    change_order: int

    def __init__(self, **kwargs: str) -> None:
        for key in self.__annotations__:
            setattr(self, key, ord(kwargs.get(key, "")))


@dataclass
class _Config:
    calendars: list[Calendar]
    ui: UIConfig
    columns: list[Columns]
    keybindings: KeyBindings

    @classmethod
    def from_dict(cls, data: dict[str, Any], default_data: dict[str, Any]) -> "_Config":
        ui_data = dict()
        ui_data.update(default_data.get("ui", {}))
        ui_data.update(data.get("ui", {}))
        ui = UIConfig(**ui_data)

        columns_data = data.get("columns", default_data.get("columns", []))
        columns: list[Columns] = [Columns(**col) for col in columns_data]

        keybindings_data = dict()
        keybindings_data.update(default_data.get("keybindings", {}))
        keybindings_data.update(data.get("keybindings", {}))
        keybindings = KeyBindings(**keybindings_data)

        calendars_data = data.get("calendars", default_data.get("calendars", {}))
        calendars: list[Calendar] = [
            Calendar(i, **cal) for i, cal in enumerate(calendars_data)
        ]
        if len(calendars) == 0:
            raise Exception("No calendars defined in configuration file")

        default_calendar = next(
            filter(lambda cal: cal.default, calendars), calendars[0]
        )
        ui.default_calendar = (
            default_calendar.name if default_calendar is not None else None
        )
        return cls(calendars=calendars, ui=ui, columns=columns, keybindings=keybindings)


def _init_config() -> _Config:
    print("Parsing configuration file...")
    xdg_config_home = os.environ.get("XDG_CONFIG_HOME") or os.path.join(
        os.path.expanduser("~"), ".config"
    )
    config_file_path = Arguments.config or os.path.join(
        xdg_config_home, "cursedtodo/config.toml"
    )
    config_file = pathlib.Path(config_file_path)
    default_config_file = pathlib.Path(
        os.path.join(pathlib.Path(__file__).parent.resolve(), "default_config.toml")
    )

    if not config_file.exists():
        if Arguments.config is None:
            print(
                "Configuration file not found, creating default configuration file..."
            )
            shutil.copy(default_config_file, config_file)
        else:
            raise Exception(f"Configuration file not found: {config_file}")

    with config_file.open("rb") as file:
        data = tomllib.load(file)

    with default_config_file.open("rb") as file:
        default_data = tomllib.load(file)

    return _Config.from_dict(data, default_data)



from __future__ import annotations
from dataclasses import dataclass
from datetime import datetime

from ics import Todo as IcsTodo
from ics.parsers.parser import ContentLine

from cursedtodo.models.calendar import Calendar
from cursedtodo.utils.time import TimeUtil


@dataclass
class Todo:
    id: int | str
    calendar: Calendar
    summary: str
    description: str
    categories: list[str] | None
    path: str | None
    priority: int
    completed: datetime | None
    due: datetime | None
    location: str | None

    def __lt__(self, other: Todo) -> bool:
        return self.priority < other.priority

    def _add_categories(self, todo_item: IcsTodo) -> None:
        for item in todo_item.extra:
            if isinstance(item, ContentLine) and item.name == "CATEGORIES":
                item.value = ",".join(self.categories or "")
                return
        todo_item.extra.append(
            ContentLine(name="CATEGORIES", value=",".join(self.categories or ""))
        )

    def _add_attributes(self, todo_item: IcsTodo) -> None:
        todo_item.name = self.summary
        todo_item.description = self.description
        todo_item.location = self.location or ""
        todo_item.priority = self.priority
        todo_item.due = self.due
        # TODO: find a way to fix types...
        # either by using "arrow", updating ics or writing our own lib
        todo_item.completed = self.completed  # type: ignore
        self._add_categories(todo_item)

    def mark_as_done(self) -> None:
        tz = TimeUtil.get_locale_tz()
        self.completed = datetime.now(tz) if self.completed is None else None



from __future__ import annotations

from curses import (
    A_BOLD,
)
from datetime import datetime
from typing import TYPE_CHECKING


from cursedtodo.config import Config
from cursedtodo.utils.colors import RED
from cursedtodo.utils.formater import Formater
from cursedtodo.utils.time import TimeUtil
from cursedtodo.utils.window_utils import add_borders, draw_line
from cursedtodo.views.base_view import BaseView

if TYPE_CHECKING:
    from cursedtodo.controlers.view_todo_controller import ViewTodoController


class ViewTodoView(BaseView):
    def __init__(self, controller: ViewTodoController) -> None:
        super().__init__(controller)
        self.controller = controller

    def render(self) -> None:
        self.height, self.length = self.window.getmaxyx()
        todo = self.controller.todo
        self.window.erase()
        self.window.border()
        add_borders(self.window)
        self.window.addstr(self.height - 1, 5, " q: go back ")
        self.window.addstr(0, 5, todo.summary)

        line = 1

        self.window.addstr(line, 1, "Liste: ", A_BOLD)
        self.window.addstr(todo.calendar.name, todo.calendar.color_attr)
        line += 1

        self.window.addstr(line, 1, "Priority: ", A_BOLD)
        text, color = Formater.formatPriority(todo.priority)
        self.window.addstr(text, color)
        line += 1

        if todo.completed:
            self.window.addstr(line, 1, "Completed: ", A_BOLD)
            self.window.addstr(f"{todo.completed.strftime(Config.ui.date_format)}")
            line += 1

        if todo.due:
            self.window.addstr(line, 1, "Due: ", A_BOLD)
            local_tz = TimeUtil.get_locale_tz()
            color = (
                RED if todo.due.replace(tzinfo=local_tz) < datetime.now(local_tz) else 0
            )
            self.window.addstr(f"{todo.due.strftime(Config.ui.date_format)}", color)
            line += 1

        if todo.categories and len(todo.categories) > 0:
            self.window.addstr(line, 1, "Categories: ", A_BOLD)
            self.window.addstr(", ".join(todo.categories or []))
            line += 1

        if todo.location and len(todo.location) > 0:
            self.window.addstr(line, 1, "Location: ", A_BOLD)
            self.window.addstr(todo.location)
            line += 1

        if todo.description:
            draw_line(self.window, line, self.length)
            line += 1
            self.window.addstr(line, 1, "Description: ", A_BOLD)
            line += 1
            i = 0
            for i, description_line in enumerate(todo.description.splitlines()):
                self.window.addstr(line + i, 1, description_line)
            line = line + i

        self.window.refresh()

    def main_loop(self) -> None:
        while True:
            k = self.window.getch()
            if self.controller.handle_key(k):
                break



from curses import window
import curses

from cursedtodo.config import Config


def add_borders(window: window) -> None:
    max_y, max_x = window.getmaxyx()
    # Can't directly use chars in box() or border() beceause of utf8
    try:
        window.box()
        if Config.ui.rounded_borders:
            window.addch(0, max_x - 1, "╮")
            window.addch(0, 0, "╭")
            window.addch(max_y - 1, 0, "╰")
            window.addch(max_y - 1, max_x - 1, "╯")
    except Exception:
        pass


def draw_line(window: window, y: int, max_x: int) -> None:
    window.addch(y, 0, "├")
    window.hline(curses.ACS_HLINE, max_x - 2)
    window.addch(y, max_x - 1, "┤")




from abc import ABC, abstractmethod
from collections.abc import Callable
from curses import window
from datetime import datetime

class BaseField(ABC):
    def __init__(
        self,
        y: int,
        window: window,
        name: str,
        id: str,
        validator: Callable[[int | str], int | str],
        value: str | list[str] | int | datetime | None = None,
    ):
        # Positional attributes
        self.window = window
        self.y = y

        # Field identification
        self.name = name
        self.id = id

        # Field value and validation
        self.value = value
        self.validator = validator

    @abstractmethod
    def _validator(self, ch: int | str) -> int | str:
        """Validate input for the field."""
        pass

    @abstractmethod
    def render(self) -> None:
        """Render the field's label and input area."""
        pass

    @abstractmethod
    def focus(self) -> None:
        """Focus on the field to allow user input."""
        pass




from curses import curs_set, window, wrapper
import curses
import sys


def app(stdscreen: window) -> None:
    curs_set(0)
    curses.use_default_colors()
    # Dynamic import to delay curses routines until the screen is fully initialized
    from cursedtodo.utils.router import Router

    router = Router(stdscreen)
    router.route_main()


def main() -> None:
    try:
        wrapper(app)
    except KeyboardInterrupt:
        sys.exit(0)


if __name__ == "__main__":
    main()



from datetime import datetime
from os import name
from typing import Any
from cursedtodo.config import Config
from cursedtodo.controlers.base_controller import Controller
from curses import KEY_RESIZE

from cursedtodo.models.todo import Todo
from cursedtodo.models.todo_repository import TodoRepository
from cursedtodo.utils.formater import Formater
from cursedtodo.utils.validator import Validator
from cursedtodo.views.edit_todo_view import EditTodoView


class EditTodoController(Controller):
    def run(self, todo: Todo | None = None) -> None:
        self.todo = todo
        self.view = EditTodoView(self)
        self.window.clear()
        self.view.render()
        self.view.main_loop()

    def handle_key(self, key: int) -> bool:
        if key == KEY_RESIZE:
            self.view.render()
        if key == 27:
            return True
        return False

    def create_or_update_todo(self, data: dict[str, Any]) -> None:
        summary = Validator.validate(data.get("summary"), str)
        description = Validator.validate(data.get("description"), str)
        location = Validator.validate(data.get("location"), str)
        categories = Validator.validate_list(data.get("categories"), str)
        list = data.get("list", "")
        priority = Formater.parse_priority(data.get("priority"))
        if data.get("due_checked"):
            due = Validator.validate_optional(data.get("due"), datetime)
        else:
            due = None
        calendar = next(filter(lambda cal: cal.name == list, Config.calendars))

        if self.todo is None:
            todo = Todo(
                "",
                calendar,
                summary,
                description,
                categories,
                None,
                priority,
                None,
                due,
                location,
            )
        else:
            todo = self.todo
            todo.calendar = calendar
            todo.summary = summary
            todo.description = description
            todo.categories = categories
            todo.due = due
            todo.location = location
            todo.priority = priority
        TodoRepository.save(todo)



import curses
from typing import Callable


class CheckboxInput:
    def __init__(
        self,
        stdscr: curses.window,
        validator: Callable[[int | str], int | str],
        value: bool = False,
    ) -> None:
        self.validator = validator
        self.lines, self.cols = stdscr.getmaxyx()
        self.stdscr = stdscr
        self.value = value

    def set_value(self, value: bool) -> None:
        self.value = value

    def gather(self) -> bool:
        return self.value

    def render(self, active: bool = False) -> None:
        self.stdscr.erase()
        if self.value:
            self.stdscr.addstr("[x]", curses.A_REVERSE if active else curses.A_NORMAL)
        else:
            self.stdscr.addstr("[ ]", curses.A_REVERSE if active else curses.A_NORMAL)

    def main(self) -> None:
        curses.curs_set(False)
        self.stdscr.keypad(True)
        while True:
            self.render(True)
            k = self.stdscr.get_wch()
            if not self.validator(k):
                break
            elif k == " ":
                self.value = not self.value
            elif k == "\x0e" or k == "\x1b" or k == "\t":
                break
        self.stdscr.chgat(0, 0, self.cols - 1, curses.A_NORMAL)
        self.stdscr.refresh()



from dataclasses import dataclass

from cursedtodo.utils.colors import BLUE, CYAN, GREEN, MAGENTA, RED, WHITE, YELLOW


@dataclass
class Priority:
    index: int
    value: str
    color: int


class Formater:
    priorities = [
        Priority(0, "No priority", WHITE),
        Priority(9, "Very Low", BLUE),
        Priority(8, "Low", CYAN),
        Priority(7, "Below Average", GREEN),
        Priority(6, "Average", YELLOW),
        Priority(5, "Above Average", MAGENTA),
        Priority(4, "High", RED),
        Priority(3, "Very High", RED),
        Priority(2, "Highest", RED),
        Priority(1, "Critical", RED),
    ]

    @staticmethod
    def formatPriority(priority: int) -> tuple[str, int]:
        # Ensure the priority is within the valid range
        if priority < 0 or priority > 9:
            raise ValueError("Priority must be between 0 and 9")

        # Get the word and color for the given priority
        fmt_priority = next(p for p in Formater.priorities if p.index == priority)
        return fmt_priority.value, fmt_priority.color

    @staticmethod
    def parse_priority(string: str | None) -> int:
        if string is None:
            return 0
        return next(p for p in Formater.priorities if p.value == string).index or 0



from glob import glob
from os import path
import os
from uuid import uuid1

from ics import Calendar, Todo as IcsTodo
from cursedtodo.config import Config
from cursedtodo.models.todo import Todo


class TodoRepository:
    @staticmethod
    def get_list(show_completed: bool = False, asc: bool = False) -> list[Todo]:
        todos: list[Todo] = []
        for calendar in Config.calendars:
            calendar_dir = os.path.expanduser(calendar.path)
            ics_files = glob(path.join(calendar_dir, "*.ics"))

            events_todos = [
                Todo(
                    event.uid,
                    calendar,
                    event.name or "",
                    event.description or "",
                    [
                        cat.strip()
                        for x in event.extra
                        if x.name == "CATEGORIES"
                        for cat in getattr(x, "value", "").split(",")
                    ]
                    or [],
                    ics_file,
                    event.priority or 0,
                    event.completed.datetime if event.completed is not None else None,
                    event.due.datetime if event.due is not None else None,
                    event.location,
                )
                for ics_file in ics_files
                for event in Calendar(open(ics_file).read()).todos
                if event.completed is None or show_completed
            ]

            todos.extend(sorted(events_todos, reverse=not asc))
        return todos

    @staticmethod
    def get_lists_names() -> list[str]:
        return [cal.name for cal in Config.calendars]

    @staticmethod
    def save(todo: Todo) -> None:
        if todo.path is None:
            calendar = Calendar()
            todo_item = IcsTodo()
            local_calendar_path = todo.calendar.path
            os.makedirs(local_calendar_path, exist_ok=True)
            todo.path = os.path.join(local_calendar_path, f"{uuid1()}.ics")
        else:
            with open(todo.path, "r") as f:
                calendar = Calendar(f.read())
            todo_item = calendar.todos.pop()
            if todo_item is None:
                raise Exception("Todo cannot be opened")

        todo._add_attributes(todo_item)
        calendar.todos.add(todo_item)
        with open(todo.path, "w") as f:
            f.writelines(calendar.serialize_iter())

    @staticmethod
    def delete(todo: Todo) -> None:
        if todo.path is None:
            raise Exception(f"Cannot delete {todo.summary} because path is null")
        os.remove(todo.path)



from curses import (
    COLOR_WHITE,
    COLOR_BLUE,
    COLOR_CYAN,
    COLOR_GREEN,
    COLOR_MAGENTA,
    COLOR_RED,
    COLOR_YELLOW,
    color_pair,
    init_pair,
)

init_pair(11, COLOR_WHITE, -1)
init_pair(12, COLOR_BLUE, -1)
init_pair(13, COLOR_CYAN, -1)
init_pair(14, COLOR_GREEN, -1)
init_pair(15, COLOR_YELLOW, -1)
init_pair(16, COLOR_MAGENTA, -1)
init_pair(17, COLOR_RED, -1)


WHITE = color_pair(11)
BLUE = color_pair(12)
CYAN = color_pair(13)
GREEN = color_pair(14)
YELLOW = color_pair(15)
MAGENTA = color_pair(16)
RED = color_pair(17)

COLORS = [BLUE, CYAN, GREEN, YELLOW, MAGENTA, RED]


def get_color(name: str | None) -> int:
    if name is None:
        return WHITE
    name = name.upper()
    if name == "WHITE":
        return WHITE
    elif name == "BLUE":
        return BLUE
    elif name == "CYAN":
        return CYAN
    elif name == "GREEN":
        return GREEN
    elif name == "YELLOW":
        return YELLOW
    elif name == "MAGENTA":
        return MAGENTA
    elif name == "RED":
        return RED
    else:
        return WHITE


def random_color(string: str) -> int:
    hashed_value = hash(string)
    index = hashed_value % len(COLORS)
    return COLORS[index]



import curses
from typing import Callable


class Cursor:
    def __init__(self, values_nb: int, loop: bool) -> None:
        self.value = 0
        self.values_nb = values_nb - 1
        self.loop = loop

    def left(self) -> None:
        if self.value > 0:
            self.value -= 1
        elif self.loop:
            self.value = self.values_nb

    def right(self) -> None:
        if self.value < self.values_nb:
            self.value += 1
        elif self.loop:
            self.value = 0


class SelectInput:
    def __init__(
        self,
        stdscr: curses.window,
        values: list[str],
        validator: Callable[[int | str], int | str],
        loop: bool = True
    ) -> None:
        self.validator = validator
        self.lines, self.cols = stdscr.getmaxyx()
        self.stdscr = stdscr
        self.values = values
        self.cursor = Cursor(len(self.values), loop)
        self.cursor.value = 0

    def set_value(self, value: str) -> None:
        try:
            if value not in self.values:
                self.cursor.value = min(int(value), len(self.values)-1)
            else:
                self.cursor.value = self.values.index(value)
        except ValueError:
            self.values.append(value)
            self.cursor.value = len(self.values) - 1
        except IndexError:
            self.cursor.value = 0

    def gather(self) -> str:
        return self.values[self.cursor.value]

    def render(self, active: bool = False) -> None:
        cursor = self.cursor
        self.stdscr.erase()
        self.stdscr.addstr(0, 0, "< ")
        self.stdscr.addstr(f"{self.values[cursor.value]}", curses.A_REVERSE if active else curses.A_NORMAL)
        self.stdscr.addstr(" >")

    def main(self) -> None:
        cursor = self.cursor
        curses.curs_set(False)
        self.stdscr.keypad(True)
        while True:
            self.render(True)
            k = self.stdscr.get_wch()
            if not self.validator(k):
                break
            if k == curses.KEY_LEFT or k == "h":
                cursor.left()
            elif k == curses.KEY_RIGHT or k == "l":
                cursor.right()
            elif k == "\x0e" or k == "\x1b" or k == "\t":
                break
        self.stdscr.chgat(0, 0, self.cols - 1, curses.A_NORMAL)
        self.stdscr.refresh()



from curses import KEY_RESIZE

from cursedtodo.config import Config
from cursedtodo.controlers.base_controller import Controller
from cursedtodo.models.todo_repository import TodoRepository
from cursedtodo.views.dialog import Dialog
from cursedtodo.views.main_view import MainView


class MainController(Controller):
    def run(self) -> None:
        self.show_completed = False
        self.asc = False
        self.view = MainView(self)
        self.window.clear()
        self.get_data()
        self.view.render()
        self.view.main_loop()

    def get_data(self) -> None:
        self.data = TodoRepository.get_list(self.show_completed)

    def handle_key(self, key: int) -> bool:
        if key == 10:
            self.router.route_view_todo(self.data[self.view.selected])
            self.data = TodoRepository.get_list(self.show_completed, self.asc)
            self.view.render()
        if key == Config.keybindings.edit:
            self.router.route_edit_todo(self.data[self.view.selected])
            self.view.render()
        if key == KEY_RESIZE:
            self.view.render()
        if key == ord("q"):
            return True
        if key == Config.keybindings.show_completed:
            self.view.selected = 0
            self.view.index = 0
            self.show_completed = not self.show_completed
            self.data = TodoRepository.get_list(self.show_completed, self.asc)
        if key == Config.keybindings.change_order:
            self.view.selected = 0
            self.view.index = 0
            self.asc = not self.asc
            self.data = TodoRepository.get_list(self.show_completed, self.asc)
        if key == Config.keybindings.new:
            self.router.route_create_todo()
            self.data = TodoRepository.get_list(self.show_completed, self.asc)
            self.view.render()
        if key == 32:
            todo = self.data[self.view.selected]
            confirmed = (
                True
                if not Config.ui.confirm_mark_as_done
                else Dialog.confirm(
                    self.window,
                    f'This action will mark "{todo.summary}" as {"done" if todo.completed is None else "pending"}. Are you sure you want to proceed?',
                    self.view.render,
                )
            )
            if confirmed:
                todo.mark_as_done()
                TodoRepository.save(todo)
                self.data = TodoRepository.get_list(self.show_completed, self.asc)
                if not self.show_completed and todo.completed is not None:
                    self.data.append(todo)
                self.view.selected = 0
                self.view.render()
        if key == Config.keybindings.delete:
            todo = self.data[self.view.selected]
            confirmed = Dialog.confirm(
                self.window,
                f'This action will permanently delete "{todo.summary}". Are you sure you want to proceed?',
                self.view.render,
            )
            if confirmed:
                TodoRepository.delete(todo)
                self.data = TodoRepository.get_list(self.show_completed, self.asc)
            self.view.render()
        return False



from datetime import datetime
import os
from zoneinfo import ZoneInfo

from cursedtodo.config import Config


class TimeUtil:
    @staticmethod
    def get_locale_tz() -> ZoneInfo:
        local_tz_path = os.readlink("/etc/localtime")
        local_tz_name = local_tz_path.split("/usr/share/zoneinfo/")[-1]
        return ZoneInfo(local_tz_name)

    @staticmethod
    def datetime_format(datetime: datetime) -> str:
        return datetime.strftime(Config.ui.date_format)

    @staticmethod
    def parse_to_datetime(string: str) -> datetime:
        format = Config.ui.date_format
        while len(format) > 1:
            try:
                return datetime.strptime(string, format)
            except ValueError:
                pass
            format = format[: len(format) - 1]
        raise Exception("Date error")



from curses import A_NORMAL, A_STANDOUT, KEY_RESIZE, window
from typing import Callable


from cursedtodo.utils.window_utils import add_borders, draw_line


class Dialog:
    @staticmethod
    def confirm(window: window, text: str, on_resize: Callable) -> bool:
        window = window
        text = text
        height, length = window.getmaxyx()
        dialog_width = max(len(text) + 4, 50)
        dialog = window.derwin(
            6,
            dialog_width,
            (height // 2) - 3,
            (length // 2) - (dialog_width // 2),
        )
        dialog.box()
        add_borders(dialog)
        dialog.addstr(1, 1, text.center(dialog_width - 2))
        draw_line(dialog, 3, dialog_width)
        dialog.addstr(4, dialog_width - 9 - 9, "[ Ok ]")
        dialog.addstr(4, dialog_width - 2 - 9, "[ Cancel ]")
        index = 1
        dialog.chgat(4, dialog_width - 2 - 6 - 10, 6, A_STANDOUT)
        while True:
            k = dialog.getch()
            if k == KEY_RESIZE:
                on_resize()
                return Dialog.confirm(window, text, on_resize)
            elif k == 9 or k == ord("h") or k == ord("l"):
                dialog.chgat(4, 1, dialog_width - 2, A_NORMAL)
                if index == 0:
                    dialog.chgat(4, dialog_width - 2 - 6 - 10, 6, A_STANDOUT)
                else:
                    dialog.chgat(4, dialog_width - 11, 10, A_STANDOUT)
                index = index ^ 1
            elif k == 10:
                return index == 1
            elif k == ord("q"):
                return False
